width  = 40.0;
height =  5.0;
depth  = 20.0;

hinge_diameter = height;
bar_diameter = height / 3;

gap  = 0.3;
gap2 = gap * 2;

unit_width = width / 5;

a();
b();

module a() {
    difference() {
        cube([width, depth - gap / 2, height]);

        translate([0.0, depth, height])
        neg();
    }
    translate([0.0, depth, height])
    pos();

    module neg() {
        for (i = [1, 3]) {
            translate([unit_width * i, 0.0, 0.0])
            rotate([0.0, 90.0, 0.0])
            cylinder(d = hinge_diameter + gap2, h = unit_width, $fn = 360);
        }
    }

    module pos() {
        for (i = [0, 2, 4]) {
            translate([unit_width * i, 0.0, 0.0])
            unit();
        }

        bar();

        module bar() {
            rotate([0.0, 90.0, 0.0])
            cylinder(d = bar_diameter, h = width, $fn = 360);
        }

        module unit() {
            rotate([0.0, 90.0, 0.0])
            linear_extrude(unit_width)
            sketch();
        }

        module sketch() {
            r = hinge_diameter / 2;

            union() {
                difference() {
                    circle(r = r, $fn = 360);

                    polygon([
                        [0, 0],
                        [0, r],
                        [r, r],
                        [r, 0],
                    ]);
                }

                polygon([
                    [      0, -gap],
                    [r + gap, -gap],
                    [      0,    r],
                ]);
            }
        }
    }
}

module b() {
    translate([0.0, depth, 0.0])
    union() {
        difference() {
            translate([0, gap / 2, 0])
            cube([width, depth - gap / 2, height]);

            translate([0, 0, height])
            neg();
        }
        translate([0.0, 0.0, height])
        pos();
    }

    module neg() {
        for (i = [0, 2, 4]) {
            translate([unit_width * i, 0.0, 0.0])
            rotate([0.0, 90.0, 0.0])
            cylinder(d = hinge_diameter + gap2, h = unit_width, $fn = 360);
        }

        rotate([0.0, 90.0, 0.0])
        // Can't use the exact hinge parameter, or there'll be problems with the
        // geometry (not a valid 2-manifold).
        cylinder(d = hinge_diameter - 0.01, h = width, $fn = 360);
    }

    module pos() {
        for (i = [1, 3]) {
            translate([unit_width * i + gap, 0.0, 0.0])
            unit();
        }

        module unit() {
            rotate([0.0, 90.0, 0.0])
            linear_extrude(unit_width - gap * 2)
            sketch();
        }

        module sketch() {
            o = hinge_diameter / 2;     // outer radius
            i = bar_diameter / 2 + gap; // inner radius

            difference() {
                // Outer structure
                union() {
                    difference() {
                        circle(r = o, $fn = 360);

                        polygon([
                            [0, gap],
                            [0,  -o],
                            [o,  -o],
                            [o, gap],
                        ]);
                    }

                    polygon([
                        [      0, gap],
                        [      0,  -o],
                        [o + gap, gap],
                    ]);
                }

                // Inner structure
                union() {
                    // Inner circle
                    circle(r = i, $fn = 360);

                    // Inner 45° overhangs
                    a = i * sqrt(2);
                    x = a - i;
                    polygon([
                        [-a,  0],
                        [-x,  i],
                        [ 0,  i],
                        [ 0, -i],
                        [-x, -i],
                    ]);
                }
            }
        }
    }
}
